////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv;

import javax.swing.*;
import java.awt.event.*;
import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.event.UndoableEditEvent;

import edu.uiuc.jdbv.diagram.Diagram;
import edu.uiuc.jdbv.gui.*;
import edu.uiuc.jdbv.schema.*;

/**
 * Defines the action for adding the currently-selected table to a diagram
 *
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.7 $
 */
public class AddTableAction extends AbstractAction
{
    /** The short descriptive text of this action = "New Diagram" */
    public static final String NAME_VALUE = "Add Table";

    /** The mnemonic keycode = KeyEvent.VK_I */
//     public static final int MNEMONIC_CODE = KeyEvent.VK_N;

    /** The long description text of this action */
    public static final String LONG_DESCRIPTION_VALUE = 
        "Adds the currently selected table from the diagram.";

    /** The statusbar message for prompting the user to click on the diagram */
    public static final String CLICK_LOCATION_STATUS =
        "Click on the diagram where the table should be added.";

    /** The statusbar message for announcing the table addition succeeded */
    public static final String TABLE_ADDED_STATUS =
        "Table added.";

    /** The statusbar message for announcing the table addition failed */
    public static final String TABLE_NOT_ADDED_STATUS =
        "Table not added.";


    /** The JDBV application instance that created this ImportSchemaAction */
    private JDBV jdbv;



    /** Constructor that takes the JDBV application instance */
    public AddTableAction( JDBV jdbv)
    {
        super( NAME_VALUE );
        this.jdbv = jdbv;

        // Setup the action parameters
//         putValue( MNEMONIC_KEY, new Integer( MNEMONIC_CODE ));
        putValue( LONG_DESCRIPTION, LONG_DESCRIPTION_VALUE );

    }


    /** Adds a tabel to the specified diagram
     * @param diagram the diagram the table should be added to
     * @param table the table being added
     */
    public void addTable( Diagram diagram, Table table )
    {
	StatusBar statbar = JDBV.getInstance().MAIN_WINDOW.getStatusBar();
	if( diagram.addTable( table )) {
	    Object cell = diagram.getCellForTable( table );
	    diagram.setSelectionCell( cell );
		
	    statbar.setStatus( TABLE_ADDED_STATUS );
	} else {
	    statbar.setStatus( TABLE_NOT_ADDED_STATUS );
	}
    }

    /** Handles the initiation of this action */
    public void actionPerformed( ActionEvent e )
    {
        Diagram currentDiagram = jdbv.getCurrentDiagram();
        if( currentDiagram != null ) {
            StatusBar statbar = JDBV.getInstance().MAIN_WINDOW.getStatusBar();
            TableList tableList = jdbv.MAIN_WINDOW.getTableList();
            Table item = (Table)tableList.getSelectedValue();
            if( item == null ) {
                System.err.println( "ERROR: No table selected to add!" );
                return;
            }

            System.out.println( "Adding table " + item.getName() );
            statbar.setStatus( CLICK_LOCATION_STATUS );
	    addTable(currentDiagram, item);

	    jdbv.undoableEditHappened
		( new UndoableEditEvent( this, new UndoAddTable
					 ( currentDiagram, item )));
        }
        
    }


    /** Class which is used to undo table additions */
    public class UndoAddTable extends AbstractUndoableEdit
    {
	private Table table;
	private Diagram diagram;

	public UndoAddTable(Diagram diagram, Table table)
	{
	    this.table = table;
	    this.diagram = diagram;
	}

	public String getPresentationName()
	{
	    return NAME_VALUE + ": " + table.getName();
	}
	
	public void redo()
	{
	    super.redo();
	    addTable( diagram, table );
	}


	public void undo()
	{
	    super.undo();
	    StatusBar statbar = JDBV.getInstance().MAIN_WINDOW.getStatusBar();
	    statbar.setStatus( RemoveTableAction.TABLE_REMOVED_STATUS +
			       table.getName() );
	    diagram.removeTable( table );
	}
    }

}




