////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv;

import edu.uiuc.jdbv.gui.*;
import edu.uiuc.jdbv.imports.*;
import edu.uiuc.jdbv.schema.*;
import edu.uiuc.jdbv.diagram.*;
import com.jgraph.event.GraphModelEvent.GraphModelChange;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.URL;
import org.apache.log4j.*;

/**
 * Starting point of the JDBV application.  It sets itself up as a singleton
 * object that can be retrieved by any of the other application's objects.
 *
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.32 $
 */
public class JDBV implements UndoableEditListener
{
    /** The logger to be used for the file = jdbv.JDBV */
    private static final Logger logger = Logger.getLogger("jdbv.JDBV");

    /** The default configuration filename = resources/import.properties */
    public static final String DEFAULT_CONFIG_FILENAME = 
        "resources/import.properties";
    
    /** The default logging file = resources/logging.cfg */
    public static final String DEFAULT_LOGGING_FILENAME = 
	"resources/logging.cfg";

    /** The version string */
    public static final String VERSION = "Construction3_05/05/03";

    /** The user-friendly program name */
    public static final String PROGRAM_NAME = "dbViZ";


    /** The filename for the logo image = "resources/splash.jpg" */
    public static final String LOGO_FILENAME = 
        "resources/splash.jpg";



    /** The action that initates exiting the application */
    public final Action EXIT_ACTION;

    /** The action that displays help about the application */
    public final Action HELP_ABOUT_ACTION;

    /** The action that initiates importing a schema */
    public final Action IMPORT_SCHEMA_ACTION;

    /** The action that initiates creating a new diagram */
    public final Action NEW_DIAGRAM_ACTION;

    /** The action that closes the current diagram */
    public final CloseDiagramAction CLOSE_DIAGRAM_ACTION;

    /** The action that removes a table from the diagram */
    public final RemoveTableAction REMOVE_TABLE_ACTION;

    /** The action that saves a diagram */
    public final SaveDiagramAction SAVE_DIAGRAM_ACTION;

    /** The action that saves a diagram in some other file*/
    public final SaveAsDiagramAction SAVE_AS_DIAGRAM_ACTION;

    /** The action that loads a diagram */
    public final LoadDiagramAction LOAD_DIAGRAM_ACTION;

    /** The action that adds a table from the diagram */
    public final AddTableAction ADD_TABLE_ACTION;

    /** The action that undoes events */
    public final UndoAction UNDO_ACTION;

    /** The action that redoes events */
    public final RedoAction REDO_ACTION;

    /** The action that auto arranges tables events */
    public final AutoArrangeAction AUTO_ARRANGE_ACTION;

    /** The action zooms in on the diagram */
    public final ZoomInAction ZOOM_IN_ACTION;

    /** The action zooms out of the diagram */
    public final ZoomNormalAction ZOOM_NORMAL_ACTION;

    /** The action zooms out of the diagram */
    public final ZoomOutAction ZOOM_OUT_ACTION;

    /** the action for printing the diagram */
    public final PrintAction PRINT_ACTION;

    /** The aciton for formatting the pages to be printed */
    public final PageSetupAction PAGE_SETUP_ACTION;

    /** The action for bringing up the print preview frame */
    public final PrintPreviewAction PRINT_PREVIEW_ACTION;

    /** Import Registry for Data sources supported by this version of JDBV */
    public final ImportRegistry IMPORT_REGISTRY;

    /** The Import definition chooser window */
    public final ImportRegistryWindow IMPORT_WINDOW;

    /** The main GUI window for this application */
    public final JDBVWindow MAIN_WINDOW;

    /** The splash window */
    private SplashWindow splashWindow = null;

    /** the singleton instance */
    private static JDBV singleton;



    /** Default constructor */
    public JDBV()
    {
        this( DEFAULT_CONFIG_FILENAME );
    }


    /** Constructor that takes the name of the configuration file */
    public JDBV( String configFilename )
    {
        showSplashScreen();

        EXIT_ACTION = new ExitAction( this );
        HELP_ABOUT_ACTION = new HelpAboutAction( this );
        IMPORT_SCHEMA_ACTION = new ImportSchemaAction( this );
        CLOSE_DIAGRAM_ACTION = new CloseDiagramAction( this );
        NEW_DIAGRAM_ACTION = new NewDiagramAction( this );
        REMOVE_TABLE_ACTION = new RemoveTableAction( this );
	SAVE_DIAGRAM_ACTION = new SaveDiagramAction( this );
	SAVE_AS_DIAGRAM_ACTION = new SaveAsDiagramAction( this );
	LOAD_DIAGRAM_ACTION = new LoadDiagramAction( this );
        ADD_TABLE_ACTION = new AddTableAction( this );
        UNDO_ACTION = new UndoAction( this );
        REDO_ACTION = new RedoAction( this );
        AUTO_ARRANGE_ACTION = new AutoArrangeAction( this );
	ZOOM_IN_ACTION = new ZoomInAction( this );
	ZOOM_NORMAL_ACTION = new ZoomNormalAction( this );
	ZOOM_OUT_ACTION = new ZoomOutAction( this );
        PRINT_ACTION = new PrintAction( this );
        PAGE_SETUP_ACTION = new PageSetupAction( this );
        PRINT_PREVIEW_ACTION = new PrintPreviewAction( this );

        IMPORT_REGISTRY = new ImportRegistry( this, configFilename );
        IMPORT_WINDOW = new ImportRegistryWindow( this );
        MAIN_WINDOW = new JDBVWindow( this );

        hideSplashScreen();
    }


    /**
     * Tells the application to start 
     */
    public void start()
    {
        MAIN_WINDOW.setVisible( true );
    }


    /**
     * Displays the splash screen
     */
    public void showSplashScreen()
    {
        if( splashWindow == null ) {
            splashWindow = new SplashWindow( LOGO_FILENAME );
            splashWindow.setVisible( true );
        }
    }


    /**
     * Hides the splash screen
     */
    public void hideSplashScreen()
    {
        splashWindow.setStatus( "Ready." );
        splashWindow.setVisible( false );
        splashWindow = null;
    }


    /**
     * Retrieves the singleton instance of the JDBV application starter
     */
    public static JDBV getInstance()
    {
        return (singleton);
    }



    /**
     * Reviews the current application state and enables/disables the
     * appropriate components for the state.
     */
    public void updateComponents()
    {
        if( getCurrentDiagram() == null ) {
            NEW_DIAGRAM_ACTION.setEnabled( true );
            PRINT_PREVIEW_ACTION.setEnabled( false );
            PRINT_ACTION.setEnabled( false );
        } else {
            NEW_DIAGRAM_ACTION.setEnabled( false );
            PRINT_PREVIEW_ACTION.setEnabled( true );
            PRINT_ACTION.setEnabled( true );
        }

    }

    protected void diagramChanged()
    {
        Diagram diagram = getCurrentDiagram();
        if (diagram == null) {
            // TODO handle this
        } else {
        } // end of else
        
        updateComponents();
    }


    /** 
     * Utility function that returns the current diagram.
     */
    public Diagram getCurrentDiagram()
    {
        if (MAIN_WINDOW != null) {
            return MAIN_WINDOW.getDiagramPane().getCurrentDiagram();
        } else {
            return null;
        } // end of else
        
        
    }

    

    /**
     * Utility function for setting a message to the status bar
     */
    public void setStatus(String message)
    {
        MAIN_WINDOW.getStatusBar().setStatus(message);
    }

    /** Forwards Events to all listeners...
        replace with javax.swingEventLister? */ 
    public void undoableEditHappened(UndoableEditEvent e)
    {
        if( !( e.getEdit() instanceof GraphModelChange )){
            getCurrentDiagram().setModified( true );
            getCurrentDiagram().getUndoManager().undoableEditHappened(e);
            UNDO_ACTION.undoableEditHappened(e);
            REDO_ACTION.undoableEditHappened(e);
        }
    }


	
    /** 
     * Starting point of the application. 
     * @param args command-line arguments supplied by the JVM
     */
    public static void main( String[] args )
    {
	String loggingFileName = System.getProperty("logging");
	String importFileName = System.getProperty("imports");
        //setup the logger
        try {
	    if( loggingFileName == null)
		loggingFileName = DEFAULT_LOGGING_FILENAME;
            URL fileURL = ClassLoader.getSystemResource( loggingFileName );
            PropertyConfigurator.configure(fileURL);
        }catch( Exception e){
            System.out.println("Couldn't configure the Logger!");
            e.printStackTrace();
        }

        logger.debug( "Starting JDBV" );
        if( importFileName == null ) {
        	singleton = new JDBV();
        } else {
            singleton = new JDBV( importFileName );
        }
        singleton.start();
    }
        



}

