////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.gui;

// import java.awt.*;
// import java.awt.event.*;
// import javax.swing.*;
// import javax.swing.event.*;

import net.javaprog.ui.*;

import edu.uiuc.jdbv.schema.Connection;
import edu.uiuc.jdbv.schema.Table;
import edu.uiuc.jdbv.diagram.Diagram;
import edu.uiuc.jdbv.diagram.TableGraphCell;
import edu.uiuc.jdbv.*;

import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.event.UndoableEditEvent;

/**
 * The ConnectionListModel wraps a tableGraphCell so the connection list can be
 *  displayed in a swing list component
 *
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.4 $
 */
public class ConnectionListModel extends AbstractCheckBoxListModel
{
    /** The table cell that this model is displaying */
    private TableGraphCell tableCell = null;

    /** Cached connections for speed */
    private Connection[] connections = null;

    /** Cached diagram to retrieve whether cells are displayed */
    private Diagram diagram = null;


    /**
     * Retrieves the element at a specified index. For ListModel interface.
     *  
     * @param index the 0-based index of the item to retrieve
     * @return Object the data at the specified index
     */
    public Object getElementAt( int index )
    {
        if( tableCell != null ) {
            return (connections[index]);
        } else {
            return (null);
        }
    }



    /**
     * Returns the length of the list. For ListModel interface.
     */
    public int getSize()
    {
        if( tableCell != null ) {
            return (connections.length);
        } else {
            return (0);
        }
    }


	/**
	 * Returns whether the element at the specified position is
	 * checked.
	 *
	 * @param index the position of the element
	 * @return whether the element at <code>index</code> is checked
     * @see net/javaprog/ui/CheckBoxListModel
	 */
	public boolean isElementChecked( int index )
    {
        if( tableCell == null || diagram == null) {
            return (false);
        } else {
            Connection conn = connections[index];
            Table indT = conn.getIndependantTable();
            Table depT = conn.getDependantTable();
            return (diagram.containsTable( indT ) &&
                    diagram.containsTable( depT ));
        }
    }


	/**
	 * Checks or unchecks the element at the given position.
	 *
	 * @param index the position of the element
	 * @param checked whether the element at <code>index</code> should
     *        be checked or not
     * @see net/javaprog/ui/CheckBoxListModel
	 */
	public void setElementChecked(int index,boolean checked)
    {
        if( tableCell != null ) {
            
            // TODO This is a big hack - should add new Actions
            Connection conn = connections[index];
            Table t = conn.getIndependantTable();
            if( t == getTable()) {
                t = conn.getDependantTable();
            }

	    Action action;
	    ActionEvent ae;

            if( checked ) {
		//                 diagram.addTable( t );
// 		JDBV.getInstance().MAIN_WINDOW.repaint();
		AddTableAction addAction = JDBV.getInstance().ADD_TABLE_ACTION;
 		addAction.addTable
 		    ( diagram, t );
 		JDBV.getInstance().undoableEditHappened
		    ( new UndoableEditEvent( this, 
					      addAction.new UndoAddTable
					     ( diagram, t )));
            } else {
		diagram.setSelectionCell( diagram.getCellForTable( t ));
                ae = new ActionEvent( this, 0, "Remove Table" );
                action = JDBV.getInstance().REMOVE_TABLE_ACTION;
                action.actionPerformed( ae );
            }
        }
    }


	/**
	 * Returns the number of checked elements in this model.
	 *
	 * @return the number of checked elements
     * @see net/javaprog/ui/CheckBoxListModel
	 */
	public int getCheckedElementsCount()
    {
        if( tableCell == null ) {
            return (0);
        } else {
            // TODO This is implemented really badly, but Sid is too sleepy.
            int count = 0;
            for( int i = getSize() - 1; i >= 0; --i ) {
                if( isElementChecked( i )) {
                    count++;
                }
            }
            return (count);
        }

    }



    /**
     * Sets the table graph cell that this connection list is displaying
     */
    public void setTableCellAndDiagram( TableGraphCell tableCell,
                                        Diagram diagram )
    {
        fireIntervalRemoved( this, 0, getSize());
        this.tableCell = tableCell;
        this.diagram = diagram;
        if( tableCell != null ) {
            // We need to put both the start and end conns in one array
            Table table = tableCell.getTable();
            Connection[] startConns = table.getStartingConnections();
            Connection[] endConns = table.getEndingConnections();
            int numConnections = startConns.length + endConns.length;
            connections = new Connection[numConnections];
            System.arraycopy( startConns, 0, connections, 0, 
                              startConns.length );
            System.arraycopy( endConns, 0, connections, startConns.length, 
                              endConns.length );
        }
        fireIntervalAdded( this, 0, getSize());
    }


    /**
     * Retrieves the table whose cell is being displayed */
    public Table getTable()
    {
        if( tableCell != null ) {
            return (tableCell.getTable());
        } else { 
            return (null);
        }
    }

}

