////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.XMLEncoder;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import edu.uiuc.jdbv.*;
import edu.uiuc.jdbv.schema.*;
import edu.uiuc.jdbv.diagram.*;
import edu.uiuc.jdbv.diagram.xmlPersist.*;

/**
 * Shows the list of diagrams that are opened with tabs
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.18 $
 */
public class DiagramPane extends JTabbedPane implements PropertyChangeListener
{
    /** The logger to be used for the file = jdbv.gui.DiagramPane */
    private static final org.apache.log4j.Logger LOG = 
        org.apache.log4j.Logger.getLogger("jdbv.gui.DiagramPane");


    /** Message that's displayed when no diagrams are loaded */
    public static final String NO_DIAGRAM_MESSAGE = "No diagrams are loaded.";

    

    /** Default constructor */
    public DiagramPane( JDBV jdbv )
    {
        ChangeListener[] listeners = { jdbv.UNDO_ACTION, 
                                       jdbv.REDO_ACTION,
                                       jdbv.CLOSE_DIAGRAM_ACTION,
                                       jdbv.SAVE_DIAGRAM_ACTION,
                                       jdbv.SAVE_AS_DIAGRAM_ACTION,
                                       jdbv.AUTO_ARRANGE_ACTION,
                                       jdbv.ZOOM_IN_ACTION,
                                       jdbv.ZOOM_OUT_ACTION,
                                       jdbv.ZOOM_NORMAL_ACTION };

        for (int i = 0; i < listeners.length; i++) {
            addChangeListener( listeners[i] );
        } // end of for ()

    }


    

    /**
     * Adds a diagram to the pane.
     */
    public void addDiagram( Diagram diagram )
    {
        JScrollPane scroller = new JScrollPane( diagram );
        add( getTitleForDiagram( diagram ), scroller );
     
        diagram.addPropertyChangeListener( Diagram.MODIFIED_PROPERTY, this );
        diagram.addPropertyChangeListener( Diagram.FILENAME_PROPERTY, this );
   
        setSelectedIndex( getDiagramCount() - 1 );
    }


    /**
     * Removes a diagram from the pane.
     */
    public void removeDiagram( Diagram diagram )
    {
        for( int i = getDiagramCount() - 1; i >= 0; --i ) {
            if( getDiagramAt( i )  == diagram ) {
                remove( getComponentAt( i ));
                setSelectedIndex( Math.min( i, getDiagramCount() - 1 ));
                fireStateChanged();
                return;
            }
        }
        LOG.warn( "Diagram not found in DiagramPane: " + diagram );
    }


    /**
     * Loads a diagram from a given .dbv file.
     */
    public void loadDiagram()
    {
    }


    /**
     * Retrieves the number of diagrams that are open in this pane
     */
    public int getDiagramCount()
    {
        return (getTabCount());
    }


    /**
     * Retrieves the currently-selected diagram
     * @return the current diagram, or <code>null</code> if there is none
     */
    public Diagram getCurrentDiagram()
    {
	int i;
	if( ( i = getSelectedIndex() ) >= 0 ) 
	    return ( getDiagramAt( i ));
	else 
	    return null;
    }


    /**
     * Retrieves the diagram with the specified tab index
     */
    public Diagram getDiagramAt( int index )
    {
        JScrollPane scroller = (JScrollPane)getComponentAt( index );
        return ((Diagram)scroller.getViewport().getView());
        
    }


    /**
     * updates all diagram tab titles
     */
    public void update()
    {
        for (int i = getDiagramCount() - 1; i >= 0; --i) {
            Diagram diagram = getDiagramAt( i );
            setTitleAt( i, getTitleForDiagram( diagram ));
        } // end of for ()
        fireStateChanged();
    }


    /** 
     * Returns a title for the supplied diagram
     */
    protected static String getTitleForDiagram( Diagram diagram )
    {
        String diagramName = diagram.getName();
        if (diagram.isModified()) {
            return (diagramName + "*");
        } else {
            return (diagramName);
        } // end of else

    }


    /** for PropertyChangeListener */
    public void propertyChange( PropertyChangeEvent e )
    {
        update();
    }


    /**
     * Special Swing paint method to draw text NO_DIAGRAM_MESSAGE if
     * no diagrams are loaded.
     *
     * @param g the Graphics context that Swing passes us to draw with
     */
    protected void paintComponent( Graphics g )
    {
        // Display "No diagrams loaded"
        if( getDiagramCount() == 0 ) {
            Toolkit tk = Toolkit.getDefaultToolkit();
            Font font = getFont().deriveFont( Font.BOLD );
            FontMetrics fontMetrics = g.getFontMetrics( font );
            int messageWidth = fontMetrics.stringWidth( NO_DIAGRAM_MESSAGE );
            Dimension size = getSize();

            // Draw the message centered in the window
            g.clearRect( 0, 0, size.width, size.height );
            g.setColor( getForeground());
            g.setFont( font );
            g.drawString( NO_DIAGRAM_MESSAGE,
                          (size.width - messageWidth)/2, size.height/2 );

        } else { // Has diagram tabs. don't do anything fancy

            super.paintComponent( g );
        } 
    }

    

}


