////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.gui;

import edu.uiuc.jdbv.JDBV;
import edu.uiuc.jdbv.schema.Schema;
import edu.uiuc.jdbv.diagram.Diagram;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import java.util.*;


/**
 * This is the main GUI window for the application.
 *
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.24 $
 */
public class JDBVWindow extends JFrame
{
    /** The default window size */
    public static final Dimension DEFAULT_WINDOW_SIZE =
        new Dimension( 780, 580 );

    /** The default diagram-pane split location in pixels = 160 */
    public static final int DEFAULT_DIAGRAM_SPLIT_LOCATION = 160;

    /** The default info-pane split location in pixels = 240 */
    public static final int DEFAULT_INFO_SPLIT_LOCATION = 240;

    /** The icon filename for the frame */
    public static final String ICON_IMAGE_FILENAME = "resources/dbviz48.gif";



    /** The JDBV application this window belongs to */
    private JDBV jdbv;

    /** The Menu bar for the window */
    private JMenuBar menuBar;

    /** The Toolbar for the window */
    private JToolBar toolBar;

    /** The JDBV Status bar */
    private StatusBar statusBar;

    /** The diagram pane that manages displaying the open diagrams */
    private DiagramPane diagramPane;

    /** The JScrollPane that contains the TableListPanel */
    private JScrollPane tableListScroller;

    /** The TableList that lists all the tables in the current schema */
    private TableList tableList;

    /** The TableInfoPanel that shows info on the currently-selected table */
    private TableInfoPanel tableInfoPanel;

    /** The selection listener for all diagrams */
    private DiagramSelectionListener diagramSelectionListener;

    /** The selection listener for all diagrams */
    private DiagramMouseWheelAdapter diagramMouseWheelAdapter;


    /** Constructor that takes the owning JDBV application */
    public JDBVWindow( JDBV jdbv )
    {
        super( jdbv.PROGRAM_NAME);

        this.jdbv = jdbv;

        setDefaultCloseOperation( DO_NOTHING_ON_CLOSE );
        initGui();
        initListeners();
        setIconImage( getApplicationIconImage());

        // TODO set location and size using user preferences -----------
        setSize( DEFAULT_WINDOW_SIZE );
        moveToScreenCenter();
        // TODO end --------------
    }


    /** Initializes GUI components */
    private void initGui()
    {
        JPanel cPane = (JPanel)getContentPane();
        JSplitPane diagramSplitPane;
        JSplitPane infoSplitPane;
        JPanel tempPanel;


        menuBar = new JDBVMenuBar( jdbv );
        setJMenuBar( menuBar );

        toolBar = new JDBVToolBar( jdbv );
        cPane.add( toolBar, BorderLayout.NORTH );

        statusBar = new StatusBar();
        cPane.add( statusBar, BorderLayout.SOUTH );

        tableList = new TableList();
        tableInfoPanel = new TableInfoPanel();

        tempPanel = new JPanel( new BorderLayout());
        tempPanel.add( new JLabel( "Table List" ), BorderLayout.NORTH );
        tempPanel.add( new JScrollPane( tableList ), BorderLayout.CENTER );

        tableInfoPanel = new TableInfoPanel();

        infoSplitPane =  new JSplitPane( JSplitPane.VERTICAL_SPLIT );
        infoSplitPane.setDividerLocation( DEFAULT_INFO_SPLIT_LOCATION );
        infoSplitPane.setResizeWeight( 0.5 );
        infoSplitPane.add( tempPanel );
        infoSplitPane.add( tableInfoPanel );

        diagramPane = new DiagramPane( jdbv );
        diagramPane.addChangeListener( tableList.createChangeListener());
        diagramPane.addChangeListener( new ChangeListener() {
                public void stateChanged( ChangeEvent e )
                {
                    Diagram diagram = jdbv.getCurrentDiagram();
                    if (diagram == null) {
                        setTitle( jdbv.PROGRAM_NAME );
                    } else {
                        setTitle( diagram.getName() + " - " + 
                                  jdbv.PROGRAM_NAME ); 
                    } // end of else
                }
            });

        diagramSplitPane = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
        diagramSplitPane.setDividerLocation( DEFAULT_DIAGRAM_SPLIT_LOCATION );
        diagramSplitPane.setResizeWeight( 0.0 );
        diagramSplitPane.add( infoSplitPane );
        diagramSplitPane.add( diagramPane );

        cPane.add( diagramSplitPane, BorderLayout.CENTER );
    }

    /** Initializes additional GUI component listeners */
    private void initListeners()
    {
        addWindowListener( new WindowAdapter() {
                public void windowClosing( WindowEvent e )
                {
                    jdbv.EXIT_ACTION.actionPerformed( null );
                }
            });
        diagramSelectionListener = new DiagramSelectionListener( this );
	diagramMouseWheelAdapter = new DiagramMouseWheelAdapter( this );
        tableList.addListSelectionListener( diagramSelectionListener );
    }

    /** Center the window in the user's screen */
    private void moveToScreenCenter()
    {
        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension screenDim = tk.getScreenSize();
        Dimension thisDim = getSize();
        int x = (screenDim.width - thisDim.width ) / 2;
        int y = (screenDim.height - thisDim.height ) / 2;

        setLocation( x, y );
    }



    /**
     * Retrieves the diagram pane for this window */
    public DiagramPane getDiagramPane()
    {
        return (diagramPane);
    }

    /** Retrieves the menubar */
    public JDBVMenuBar getJDBVMenuBar()
    {
	return (JDBVMenuBar)menuBar;
    }


    /**
     * Initiates loading a saved diagram
     */
    public void handleLoadDiagram()
    {

	diagramPane.loadDiagram();
    }

    /**
     * Add the supplied diagram to the window and sets it to be the current
     * diagram.
     */
    public void addDiagram( Diagram diagram )
    {
        diagram.addGraphSelectionListener( diagramSelectionListener );
        diagram.addGraphSelectionListener( tableInfoPanel.
                                           getGraphSelectionListener());
	diagram.addMouseWheelListener( diagramMouseWheelAdapter );
        diagramPane.addDiagram( diagram );
        tableList.setDiagram( diagram );
		
    }


    /**
     * Removes the supplied diagram from the window
     */
    public void removeDiagram( Diagram diagram )
    {
        getDiagramPane().removeDiagram( diagram );

    }


    /**
     * Updates components to match the current state of the window
     */
    public void updateComponents()
    {
        // TODO stuff here!
    }


    /**
     * Retrieves the statusbar for the window.
     */
    public StatusBar getStatusBar()
    {
        return (statusBar);
    }

    /**
     * Retrieves the table list for the window
     */
    public TableList getTableList()
    {
        return (tableList);
    }


    /**
     * Retrieves the table info panel for the window
     */
    public TableInfoPanel getTableInfoPanel()
    {
        return (tableInfoPanel);
    }


    /**
     * Returns the icon image for the window
     */
    public static Image getApplicationIconImage()
    {
        return (Toolkit.getDefaultToolkit().getImage
		( ClassLoader.getSystemResource( ICON_IMAGE_FILENAME )));
    }
        
}

