////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.imports;

import edu.uiuc.jdbv.schema.*;

/**
 * Represents a Connection that is pending its 'official' creation in a schema.
 * Stores enough information that it can later be created once the schema's
 * tables are all present.
 *
 * @author Brian Sidharta
 * @version $Revision: 1.2 $
 * @see PendingConnectionCache
 */
public class PendingConnection
{
    private String pkTable;

    private String[] pkCols;

    private String fkTable;
    
    private String[] fkCols;

    private int hashCode;


    /**
     * Creates a pending connection for PrimaryKey table and ForeignKey table
     */
    public PendingConnection( String pkTable, String fkTable )
    {
        this.pkTable = pkTable;
        this.fkTable = fkTable;
        pkCols = new String[0];
        fkCols = new String[0];

        // Create hashcode
        String hashString = pkTable + "-" + fkTable;
        hashCode = hashString.hashCode();
    }


    /**
     * Adds columns involved in this pending connection
     */
    public void addColumns( String pkCol, String fkCol )
    {
        pkCols = addStringToArray( pkCol, pkCols );
        fkCols = addStringToArray( fkCol, fkCols );
    }


    /**
     * Returns whether the specified primary-key column is included in
     * this PendingConnection
     */
    public boolean containsPKColumn( String pkCol )
    {
        for (int i = pkCols.length - 1; i >= 0; --i) {
            if ( pkCol.equals( pkCols[i] )) {
                return (true);
            } // end of if ()
        } // end of for ()
        return (false);
    }

    /**
     * Utility method. Enlarges a string array and appends a string to the 
     * end of it.
     * @return the new String array with addition at the end
     */
    private static String[] addStringToArray( String addition, 
                                              String[] oldArray )
    {
        String[] newArray = new String[oldArray.length + 1];
        System.arraycopy( oldArray, 0, newArray, 0, oldArray.length );
        newArray[oldArray.length] = addition;
        return (newArray);
    }


    /**
     * @see java.lang.Object#equals
     */
    public boolean equals( Object o )
    {
        if (o instanceof PendingConnection) {
            PendingConnection oConn = (PendingConnection)o;
            return (pkTable.equals( oConn.pkTable ) &&
                    fkTable.equals( oConn.fkTable ));
        } // end of if ()
        
        return false;
    }


    /**
     * @see java.lang.Object#hashCode
     */
    public int hashCode()
    {
        return (hashCode);
    }


    /**
     * Retrieves the primary-key table name 
     */
    public String getPKTable()
    {
        return (pkTable);
    }

    /**
     * Retrieves the primary-key column names
     */
    public String[] getPKCols()
    {
        return (pkCols);
    }

    /**
     * Retrieves the foreign-key table name 
     */
    public String getFKTable()
    {
        return (fkTable);
    }

    /**
     * Retrieves the foreign-key column names
     */
    public String[] getFKCols()
    {
        return (fkCols);
    }

}

