////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
// DBImportDefinition.java
package edu.uiuc.jdbv.imports.oracle;


/*<Imports>*/
import java.io.*;
import java.util.*;
import java.sql.*;
import edu.uiuc.jdbv.imports.*;
import edu.uiuc.jdbv.schema.*;
import javax.swing.JPanel;
/*</Imports>*/

/**
 * The importSchema method connects to a database 
 * and returns a schema object
 *
 * @author Uday Kale
 * @author Brian Sidharta
 * @version $Revision: 1.9 $
 */
public class DBImportDefinition extends ImportDefinition
{
    /** The logger to be used for the file */
    private static final org.apache.log4j.Logger LOG = 
        org.apache.log4j.Logger.
        getLogger("jdbv.imports.oracle.DBImportDefinition");


    private String connString;
    private String userName;
    private String passwd;
    private String database;
    private java.sql.Connection conn;
    

    public DBImportDefinition()
    {
    }

    public void setConnString(String connstr, String user, String pass)
    {
	connString = connstr;
	userName = user.toUpperCase();
	passwd = pass;
    }

    public void setDatabase(String db)
    {
	database = db;
    }

    public String checkValidity() 
    {
	// Check if a database is selected and connection String set up 
	if (database == null) 
	    return "Database not selected";
	else if (connString == null)
	    return "Connection String not set";
	else if (userName == null)
	    return "User Name not set";

	// Make the connection based on the database selected
	if (database.equals("Oracle")) {

            LOG.debug("Connection String is: "+
                               connString+"@"+userName+":"+passwd);
            
            conn = OracleConnect.connectDB(connString, userName, passwd);
            if (conn == null)
                return "Could not connect to the database";
        }
	else
	    return "Sorry, currently can connect to only Oracle database";
        
	LOG.debug( "Connection made" );

	return (null);
    }

    /**
     *  Description describing the type of import
     */
    public String getDescription ()   {
	return "Import From Database" ;
    }

    public JPanel getGuiPanel(){
	return new DBGuiPanel(this);
    }

    public Schema importSchema() throws ImportException
    {
        Table table = null;
        PendingConnectionCache connCache = new PendingConnectionCache();
        StringBuffer schemaNameSb = new StringBuffer();
        schemaNameSb.append( userName );
        schemaNameSb.append( "@" );
        schemaNameSb.append( connString );
        Schema dbSchema = new Schema( schemaNameSb.toString());
        try {
            DatabaseMetaData dbmeta = conn.getMetaData();
            ResultSet rs = dbmeta.getTables(null, userName, "%", null);
            while (rs.next()) {
                String tableName = rs.getString( 3 );
                Table tab = new Table( tableName );
                Column col;

                // Column 3 is table name
                LOG.debug("Table "+ tableName);

                // Add columns of this table
                ResultSet cols = 
                    dbmeta.getColumns(null, userName, tableName, null);
                while (cols.next()) {
                    //  Column 4 is column name
                    //  Column 6 is type
                    col = new Column(cols.getString(4), cols.getString(6), 
                                     false);
                    tab.addColumn(col);
                    LOG.debug("- column "+ 
                              cols.getString(4) + " of type "+
                              cols.getString(6));
                }
		cols.close();

                LOG.debug("- addding table to schema");

                // Mark primary keys
                ResultSet pkRS = dbmeta.getPrimaryKeys( null, userName, 
                                                        tableName );
                while (pkRS.next()) {
                    Column pkCol = tab.getColumn( pkRS.getString( 4 ));
                    pkCol.setKey( true );
                } // end of while ()
		pkRS.close();

                // Add table to schema
                dbSchema.addTable(tab);

                // Add foreign key connections
                ResultSet foreignkeys =
                    dbmeta.getImportedKeys(null, userName, tableName);
                while(foreignkeys.next()) {
                    LOG.debug( "- f-key:  " + 
                               foreignkeys.getString( 3 ) + "." + 
                               foreignkeys.getString( 4 ) +
                               " <-- " +
                               foreignkeys.getString( 7 ) + "." + 
                               foreignkeys.getString( 8 ) + 
                               "  (" + foreignkeys.getInt( 9 ) + ")");

                    // Column 3,4 is primary table, column
                    // Column 7,8 is dependant table, column
                    String pkTable = foreignkeys.getString( 3 );
                    String pkCol = foreignkeys.getString( 4 );
                    String fkTable = foreignkeys.getString( 7 );
                    String fkCol = foreignkeys.getString( 8 );
                    connCache.addConnection( pkTable, pkCol, fkTable, fkCol );
                }
		foreignkeys.close();
            }
	    rs.close();
            connCache.createAllConnections( dbSchema );
        } 
        catch(Exception ex) 
        {
            LOG.error( "Error while processing table list.", ex);
            throw (new ImportException( "Error importing schema.", ex ));
        }


        return dbSchema;
    }

}
