////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.schema;

import java.util.*;
import edu.uiuc.jdbv.util.CollectionUtils;


/**
 * Representation of a table in a database.
 *
 * @author Ross Paul
 * @version $Revision: 1.7 $
 */
public class Table{

    /** The name of the table from the schama */
    private final String name;

    private Map columns;

    /** Connection object linking columns in OTHER tables
     *  To columns in this one.  (this table is independant)*/
    private List endingConnections;

    /** Connection object which links columns in THIS table
     *  to columns in ohter tables. (this table is dependant) */
    private List startingConnections;



    /** 
     * Tables are created with their names.  Columns are added using
     * addColumn, and Connections are added using 
     * Connection's static linkTables function 
     * @param name the table name. should Never be null
     */
    public Table(String name)
    {
        this.name = name;
        columns = new LinkedHashMap();
        endingConnections = new ArrayList();
        startingConnections = new ArrayList();
    }
    
    /** Adds the column to a hashtable keyed on column name */
    public void addColumn(Column column){
        columns.put(column.getName().toUpperCase(),column);
    }

    /** Returns the name of the Table */
    public String getName()
    {
        return name;
    }

    /** Fetches the specified Column */
    public Column getColumn(String name){
        return (Column)columns.get(name.toUpperCase());
    }

    /** Returns all of the table's columns */
    public Column[] getColumns()
    {
        Collection c = columns.values();
        Column[] columnArray = new Column[c.size()];
        return (Column[])c.toArray(columnArray);
    }

    /** returns the connections TO this this table 
     * (this table is independant) */
    public Connection[] getEndingConnections(){
        Connection[] connections = new Connection[endingConnections.size()];
        return (Connection[])endingConnections.toArray(connections);
    }

    /** returns the connections FROM this this table 
     * (this table is dependant) */
    public Connection[] getStartingConnections(){
        Connection[] connections = new Connection[startingConnections.size()];
        return (Connection[])startingConnections.toArray(connections);
    }


    /** Package level function which should only be called by 
     *  Connection.linkTables().  Adds new connection. */
    public void addEndingConnection(Connection connection){
        endingConnections.add(connection);
    }

    /** Package level function which should only be called by 
     *  Connection.linkTables().  Adds new connection. */
    public void addStartingConnection(Connection connection){
        startingConnections.add(connection);
    }
	
    /** overrides the toString method to allow logical viewing */
    public String toString(){
        StringBuffer buf = new StringBuffer();
        buf.append("\tTable: " + name);
        buf.append("\n\t   Columns -\n");
        for(Iterator i = columns.entrySet().iterator(); i.hasNext();){
            buf.append(i.next());
        }
        buf.append("\n\t   Connections -\n");
        for(Iterator i = startingConnections.iterator();i.hasNext();){
            buf.append(i.next());
        }
        for(Iterator i = endingConnections.iterator();i.hasNext();){
            buf.append(i.next());
        }
        buf.append("\n");
        return buf.toString();
    }


    /**
     * Checks if two objects are equivalent
     * @author Brian Sidharta
     */
    public boolean equals( Object o )
    {
        if( o instanceof Table ) {
            Table t = (Table)o;
            return( name.equals( t.name ) &&
                    CollectionUtils.mapsEqual( columns, t.columns ) &&
                    CollectionUtils.listsEqual( endingConnections, 
                                                t.endingConnections ) &&
                    CollectionUtils.listsEqual( startingConnections, 
                                                t.startingConnections ));
        } else {
            return (false);
        }

    }

}

