////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.util;

import edu.uiuc.jdbv.*;
import java.awt.*; 
import java.awt.event.*; 
import java.awt.image.*; 
import java.util.*; 
import java.awt.print.*; 
import javax.swing.*; 
import javax.swing.border.*; 
import javax.swing.event.*; 


/** 
 * PrintPreview takes in any Printable object and creates displays a new frame
 * showing how the printable will look when printed.  The preview also gives 
 * the user the option to change the scale in the preview rather than doing so
 * from the page setup dialog.
 *
 * @author Ross Paul
 * @version $Revision: 1.4 $
 */
public class PrintPreview extends JFrame 
{

    protected int pageWidth; 
    protected int pageHeight; 
    protected Printable target; 
    protected JComboBox scaleBox; 
    protected PreviewContainer previewContainer; 
    protected PageFormat pageFormat;
  

    public PrintPreview(Printable target) { 
        this(target, "Print Preview"); 
    }

    public PrintPreview( Printable target, String title )
    {
        this( target, title, null );
    }

    public PrintPreview( Printable target, String title, PageFormat format ) { 
        super(title); 

        pageFormat = format;
        setSize(600, 400); 
        this.target = target; 
        JToolBar toolBar = new JToolBar(); 
        JButton button = new JButton("Print", new ImageIcon("print.gif")); 

        ActionListener actionListener = new ActionListener() { 
                public void actionPerformed(ActionEvent e) { 
                    try { 
                        // Use default printer, no dialog 
                        PrinterJob prnJob = PrinterJob.getPrinterJob(); 
                        prnJob.setPrintable(PrintPreview.this.target); 
                        setCursor( Cursor.getPredefinedCursor( 
                                                              Cursor.WAIT_CURSOR)); 
                        prnJob.print(); 
                        setCursor( Cursor.getPredefinedCursor( 
                                                              Cursor.DEFAULT_CURSOR)); 
                        dispose(); 
                    }

                    catch (PrinterException ex) { 
                        ex.printStackTrace(); 
                        System.err.println("Printing error: "+ex.toString()); 
                    }
                }
            }; 

        button.addActionListener(actionListener); 
        button.setAlignmentY(0.5f); 
        button.setMargin(new Insets(4,6,4,6)); 
        toolBar.add(button); 

  

        button = new JButton("Close"); 
        actionListener = new ActionListener() { 
                public void actionPerformed(ActionEvent e) { 
                    dispose(); 
                }
            }; 

        button.addActionListener(actionListener); 
        button.setAlignmentY(0.5f); 
        button.setMargin(new Insets(2,6,2,6)); 
        toolBar.add(button); 

        String[] scales = { "10 %", "25 %", "50 %", "100 %", "150 %" }; 
        scaleBox = new JComboBox(scales); 
       
        actionListener = new ActionListener() { 
                public void actionPerformed(ActionEvent e) { 
                    Thread runner = new Thread() { 
                            public void run() 
                            { 
                                String str = scaleBox.getSelectedItem(). 
                                    toString(); 
                                if (str.endsWith("%")) 
                                    str = str.substring(0, str.length()-1); 
                                str = str.trim(); 
                                int scale = 0; 
                                try { scale = Integer.parseInt(str); } 
                                catch (NumberFormatException ex) { return; } 
                                int w = (int)(pageWidth*scale/100); 
                                int h = (int)(pageHeight*scale/100); 
                                Component[] comps = previewContainer.getComponents(); 

                                for (int k=0; k<comps.length; k++) { 
                                    if (!(comps[k] instanceof PagePreview)) 
                                        continue; 

                                    PagePreview pp = (PagePreview)comps[k]; 
                                    pp.setScaledSize(w, h); 
                                }

                                previewContainer.doLayout(); 
                                previewContainer.getParent().getParent().validate(); 
                            }
                        }; 

                    runner.start(); 
                }}; 

        scaleBox.addActionListener(actionListener); 
        scaleBox.setMaximumSize(scaleBox.getPreferredSize()); 
        scaleBox.setEditable(true); 

        toolBar.addSeparator(); 
        toolBar.add(scaleBox); 
        getContentPane().add(toolBar, BorderLayout.NORTH); 

        previewContainer = new PreviewContainer(); 

        PrinterJob prnJob = PrinterJob.getPrinterJob(); 
        if( pageFormat == null )
            pageFormat = prnJob.defaultPage(); 

        if (pageFormat.getHeight()==0 || pageFormat.getWidth()==0) { 
            System.err.println("Unable to determine default page size"); 
            return; 
        }

        pageWidth = (int)(pageFormat.getWidth()); 
        pageHeight = (int)(pageFormat.getHeight()); 

        int scale = 10; 
        int w = (int)(pageWidth*scale/100); 
        int h = (int)(pageHeight*scale/100); 
        int pageIndex = 0; 

        try { 
            while (true) 
            { 
                BufferedImage img = new BufferedImage(pageWidth, 
                                                      pageHeight, BufferedImage.TYPE_INT_RGB); 
                Graphics g = img.getGraphics(); 
                g.setColor(Color.white); 
                g.fillRect(0, 0, pageWidth, pageHeight); 

                if (target.print(g, pageFormat, pageIndex) != 
                    Printable.PAGE_EXISTS) 
                    break; 

                PagePreview pp = new PagePreview(w, h, img); 
                previewContainer.add(pp); 
                pageIndex++; 
            }
        }
        catch (PrinterException e) { 
            e.printStackTrace(); 
            System.err.println("Printing error: "+e.toString()); 
        }

        JScrollPane ps = new JScrollPane(previewContainer); 
        getContentPane().add(ps, BorderLayout.CENTER); 

        setDefaultCloseOperation(DISPOSE_ON_CLOSE); 
        setVisible(true); 
    }

    class PreviewContainer extends JPanel 
    {
        protected int H_GAP = 16; 
        protected int V_GAP = 10; 

        public Dimension getPreferredSize() { 
            int n = getComponentCount(); 
            if (n == 0) 
                return new Dimension(H_GAP, V_GAP); 

            Component comp = getComponent(0); 
            Dimension dc = comp.getPreferredSize(); 
            int w = dc.width; 
            int h = dc.height; 

            Dimension dp = getParent().getSize(); 
            int nCol = Math.max((dp.width-H_GAP)/(w+H_GAP), 1); 
            int nRow = n/nCol; 

            if (nRow*nCol < n) 
                nRow++; 

            int ww = nCol*(w+H_GAP) + H_GAP; 
            int hh = nRow*(h+V_GAP) + V_GAP; 
            Insets ins = getInsets(); 

            return new Dimension(ww+ins.left+ins.right, 
                                 hh+ins.top+ins.bottom); 
        }

        public Dimension getMaximumSize() { 
            return getPreferredSize(); 
        }

        public Dimension getMinimumSize() { 
            return getPreferredSize(); 
        }

        public void doLayout() { 
            Insets ins = getInsets(); 
            int x = ins.left + H_GAP; 
            int y = ins.top + V_GAP; 
            int n = getComponentCount(); 

            if (n == 0) 
                return; 

            Component comp = getComponent(0); 
            Dimension dc = comp.getPreferredSize(); 

            int w = dc.width; 
            int h = dc.height; 

            Dimension dp = getParent().getSize(); 
            int nCol = Math.max((dp.width-H_GAP)/(w+H_GAP), 1); 
            int nRow = n/nCol; 

            if (nRow*nCol < n) 
                nRow++; 

            int index = 0; 

            for (int k = 0; k<nRow; k++) { 
                for (int m = 0; m<nCol; m++) { 
                    if (index >= n) 
                        return; 

                    comp = getComponent(index++); 
                    comp.setBounds(x, y, w, h); 
                    x += w+H_GAP; 
                }
                y += h+V_GAP; 
                x = ins.left + H_GAP; 
            }
        }
    }

    class PagePreview extends JPanel 
    {
        protected int m_w; 
        protected int m_h; 
        protected Image m_source; 
        protected Image m_img; 

        public PagePreview(int w, int h, Image source) { 
            m_w = w; 
            m_h = h; 
            m_source= source; 

            m_img = m_source.getScaledInstance(m_w, m_h, 
                                               Image.SCALE_SMOOTH); 
            m_img.flush(); 

            setBackground(Color.white); 
            setBorder(new MatteBorder(1, 1, 2, 2, Color.black)); 
        }

        public void setScaledSize(int w, int h) { 
            m_w = w; 
            m_h = h; 
            m_img = m_source.getScaledInstance(m_w, m_h, 
                                               Image.SCALE_SMOOTH); 
            repaint(); 
        }

        public Dimension getPreferredSize() { 
            Insets ins = getInsets(); 
            return new Dimension(m_w+ins.left+ins.right, 
                                 m_h+ins.top+ins.bottom); 
        }

        public Dimension getMaximumSize() { 
            return getPreferredSize(); 
        }

        public Dimension getMinimumSize() { 
            return getPreferredSize(); 
        }

        public void paint(Graphics g) { 
            g.setColor(getBackground()); 
            g.fillRect(0, 0, getWidth(), getHeight()); 
            g.drawImage(m_img, 0, 0, this); 
            paintBorder(g); 
        }
    }
}
